
package AbstractParser;
use strict;
use warnings;

=pod 

=head1 TITLE

AbstractParser

=head1 AUTHOR

paramsan

=head1 NAME

AbstractParser: Represents the abstract interface for any parser object which 
parses weekly specials from a store.This class is only supposed to be used as 
a base/parent class for a "specific" parser. 

=head1 SYNOPSIS

	use AbstractParser;

=head1 REQUIRES

Perl5.10.0 

=head1 EXPORTS

Nothing

=head1 DESCRIPTION

AbstractParser represents an abstract interface that any class implementing  parsing 
weekly specials should inherit from in the project. As such, the child class should 
implement any abstract methods below, in particular C<GetSpecials> (L<"GetSpecials">).

Also see L<AbstractParserFactory> on how to construct concrete Parser objects.

=head1 CLASS VARIABLES

=head2 $AbstractParser::VERBOSITY_ALL

Used to set all verbose level for the parser, that is, all pertinent information,
errors, warning, informational messages will be printed.

=head2 $AbstractParser::VERBOSITY_NONE 

Used to set none verbose level for the parser, that is only errors (printed) are that will cause
the module to die()..

=cut

# ================================
# Class Global Constants
# ================================
our $VERBOSITY_ALL  = 4;
our $VERBOSITY_NONE = 0;

=head1 METHODS

=head2 new (constructor)

=over 4

=item $abstractParser = AbstractParser->new();

Creates and returns an AbstractParser.
For internal use only, an AbstractParser object 
is not supposed to be constructed.

=item Arguments

None

=item Returns 

A reference to an AbstractParser

=back

=cut	

sub new {
	my $class = shift;
	
    my $self = {
        _storeName => undef,
		_verbose      => undef
    };
	
    bless $self,  $class;
    return $self;
}

=pod

=head2 GetSpecials

=over 4

=item Description

The subclasses are supposed to implement this method.


=item Arguments 

B<$zipCode:> Zipcode for which to retrieve the specials for. 

=item Returns

This method should return A reference to an array of Specials in the following format,

[ [ItemName, ItemPrice],
  [ItemName, ItemPrice]
  ...
];

This is an abstract method and currently calls die() with the error messsage:
"Abstract method GetSpecials not implemented\n"

=item Usage

C<< $specialRef = $abstractParser->GetSpecials(); >>

=back

=cut	

sub GetSpecials {
	my $self = shift;
	my $zipCode = shift;
	die("Abstract method GetSpecials not implemented\n");
}

=pod 

=head2 GetStoreName

=over 4

=item Description

Returns the store name that this parser will parse specials for.

=item Arguments 

None

=item Returns

The storename as a string.

=item Usage

C<< $storeName = $abstractParser->GetStoreName(); >>

=back

=cut	
	
sub GetStoreName {
	my $self = shift;
	return $self->{_storeName};
}

=pod

=head2 GetVerbosityLevel

=over 4

=item Description

Returns the current verbosity level for the parser

=item Arguments 

None

=item Returns

The verbosity level. See L<Class Variables> for the different
verbosity values.

=item Usage

C<< $verbosityLevel = $abstractParser->GetVerbosityLevel(); >>

=back

=cut	

sub GetVerbosityLevel {
	my $self = shift;
	return $self->{_verbose};
}

=pod

=head2 SetVerbosityLevel

=over 4

=item Description

Sets the verbosity level for the parser.

=item Arguments 

B<$verbosityLevel> The verbosity level. See L<Class Variables> for the different verbosity values.

=item Returns

Nothing

=item Usage

C<< $abstractParser->SetVerbosityLevel($AbstractParser::VERBOSITY_ALL); >>

=back

=cut	

sub SetVerbosityLevel {
	my $self = shift;
	my $level = shift;
	
	$self->{_verbose} = $level;
}

=pod

=head2 _SetStoreName

=over 4

=item Description

Sets the store name for this parser. This is an internal method and to be used by the 
children classes.

=item Arguments 

B<$storeName:> The store name.

=item Returns

Nothing

=item Usage

C<< $abstractParser->_SetStoreName($storeName); >>

=back

=cut	

sub _SetStoreName {
	my $self = shift;
	my $storeName = shift;
	
	$self->SetStoreName = $storeName;
}

1;