#!/bin/perl
#package db_specials;

# load module
use DBI;
use strict;
use warnings;

# Sets the verbosity level in the file. 
# 0 will not cause anything other than serious error message to print
# 4 will cause everything to print.
my $verbose = 0;

# Path to the dbFile, can be set outside of this file using
# GetOrSetDbConfigFile() function.
my $dbFile = "../../website/lib/tinymvc/myapp/configs/database.php";

# The dbParams hash to store the database connection information
# See PopulateDbParamsHash() for more information.
my %dbParams = ();

# The table names used througout the file.
my $STORE_CHAIN_TABLE_NAME = "storechain";
my $STORE_LOCATION_TABLE_NAME = "storelocation";
my $SPEICAL_TABLE_NAME = "special";

# Gets or sets the value of the database config filename
# Arguments:
# $dbConfigFile: The database file [optional]
# Returns: 
# The database configuration filename used to parse the
# database access credentials.
# If the argument is passed in, the function will set the database filename
# to be the argument that is passed in and return it. Otherwise, the
# function will just return the database configuration filename.
sub GetOrSetDbConfigFile {
	my $dbConfigFile = shift;
	
	if (defined($dbConfigFile)) {
		$dbFile  = $dbConfigFile;
	}
	
	return $dbFile;
}

# Populates the %dbParams hash declared above by parsing 
# the $dbFile. The $dbFile is assumed to be in the following 
# format:
#
# $config['type'] = 'mysql';      // connection type
# $config['host'] = 'localhost';  // db hostname
# $config['name'] = 'cs1';     // db name
# $config['user'] = 'jp';     // db username
# $config['pass'] = 'password';     // db password
#
# Arguments: None
# Return: Nothing
# Post: 'aKey' in $config['aKey'] = 'aValue'  can be accessed from
# the dbParams has as $dbParams{'aKey'} (with aValue as the 
# return).
#
# The function will die() if it is unable to open the file. Other than
# that this function assumes the file is correctly formatted.
sub PopulateDbParamsHash() {
	my $line;
	open(DB_FILE, "$dbFile") or die("Cannot open $dbFile $@");
	while (defined($line = <DB_FILE>)) {
		chomp($line);
		if ($line =~ /\[\'(\w+)\'\]\s*=\s*\'(.+)\';/) {
			print $1, " : ", $2, "\n" if ($verbose > 1);
			$dbParams{$1} = $2;
		}
		print $line, "\n" if ($verbose > 2);
	}
	close(DB_FILE);
}

# A handy function to connect to the database by calling
# PopulateDbParamsHash() and connecting to the DB using
# the populated values.
# Arguments: None
# Returns: The database handle $dbh.
# The database connection is set to call die() on errors.
sub ConnectToDatabase() {
    
    # TODO:
    # If the DbParams hash is already populated
    # then don't repopulate it
    PopulateDbParamsHash();
	
    # connect
    my $dbh = DBI->connect("DBI:mysql:database=$dbParams{name};host=$dbParams{host}", 
			   "$dbParams{user}", 
			   "$dbParams{pass}", {'RaiseError' => 1});
    return $dbh;
}

# Returns an array of zipcodes of a particular storeName from the database
# Arguments: $storeName: The name of the store to get its zipcodes
# Returns: An array of all the store zipcodes with storeName.
#               An empty array is returned if there are no zipcodes corresponding
#               to storeName or there was any error (in which case the db functions
#                would probably call die()).
sub GetStoreZipCodes {

    my $storeName = shift;
    my @zipCodeArray = ();
    my $dbh = ConnectToDatabase();

    my $storeChainId = GetScalarValue($dbh, "$STORE_CHAIN_TABLE_NAME", "id", "name = ?", $storeName);

    if (defined($storeChainId)) {
		my $sth = $dbh->prepare("SELECT zipCode FROM $STORE_LOCATION_TABLE_NAME WHERE storeChainId = ?");
		$sth->execute($storeChainId);
		my $row;


		while ($row = $sth->fetchrow_array()) {
			push(@zipCodeArray, $row);
		}
		$sth->finish();
    }

    $dbh->disconnect();

    return @zipCodeArray;
}


# A function to insert specials into the database.
# The storeName under which the specials are added is passed in 
# as $storeName with $zipCode as its identifying feature. If there are 
# multiple stores at zipCode then specials will be inserted into the first 
# store at $zipCode returned by the db.
# Parameters: $storeName: Name of the store
#                     $zipCode: The zipcode of the store to add specials for.
#                     $specialsRef: A reference to specials. The specials are
#                      assumed to be array of tuples, where each tuple is 
#                       composed of (item, price) for a special.
# Returns: Nothing
# This function can die if anything goes wrong related to the database.
sub InsertSpecialsIntoDb {

	my $storeName = shift;
    my $zipCode = shift;
	my $allSpecialsRef = shift;
	
	my $rows;
	my $storeChainId;
	my $storeLocationId;
	
	# connect
	my $dbh = ConnectToDatabase(); 

	$storeChainId = GetScalarValue($dbh, "$STORE_CHAIN_TABLE_NAME", "id", "name = ?", $storeName);

 	$storeLocationId = GetScalarValue($dbh, "$STORE_LOCATION_TABLE_NAME", "id", 
						 "storeChainId = ? AND zipCode = ?", $storeChainId, $zipCode);

	if (defined($storeLocationId)) {
	    
	    my $i = 0;
	    for (my $special = $allSpecialsRef->[$i]; defined($special); $i++, $special = $allSpecialsRef->[$i]) {

		print "Inserting into db: ", $special->[0], " : ", $special->[1], "\n" if ($verbose > 3);
		my $query = qq{INSERT INTO $SPEICAL_TABLE_NAME 
					(storeLocationId, title, price) 
					 VALUES
					(?, ?, ?)};
		$rows = $dbh->do($query, undef, $storeLocationId, $special->[0], $special->[1]);
	    }
	}
	else {
	    print "******No data in perl\n";
	}

	#clean up
	$dbh->disconnect();
}

# A handly function to get the id of the last insert statement.
# Arguments: $dbh: The database handler.
# Returns: The id corresponding to the last insert statement.
# This value can be undefined if there wasn't any last insert 
# statement.
sub GetLastInsertedId {
	my $dbh = shift;
	my $sth = $dbh->prepare("SELECT LAST_INSERT_ID()");
	$sth->execute();
	my ($lastInsertedId) = $sth->fetchrow_array;
	$sth->finish();
	return $lastInsertedId;
}
	
# A handy function to get a scalar value from the database.
# Arguments: 
#                  $dbh: The database handle
#                  $tableName: The name of the table to retrieve values from.
#                  $columnName: The name of the column whose value will
#                   be retrieved.
#                  $whereCondition: The where condition for the sql statement.
#                   This string should not have "where" in it anywhere.
#                    ...: Any bind parameters used in the where statement 
#                    above
# Return:
# Returns the single scalar value of the column $columName from table
# $tableName matching the $whereCondition.
# If there are multiple rows returned by the where condition, this function will
# return the first row returned by the database. 
sub GetScalarValue {
	my $dbh = shift;
	my $tableName = shift;
	my $columnName = shift;
	my $whereCondition = shift;
	my @bind_params = @_;

	my $sth = $dbh->prepare("SELECT $columnName FROM $tableName WHERE $whereCondition");
	$sth->execute(@bind_params);
		
	my ($columnValue) = $sth->fetchrow_array;

	$sth->finish();
	return $columnValue;
}

	
# A function to insert information about a store into the database.
# Arguments:
# 				$storeName: Name of the store
# 				$storeWebsite: The website of the store
# 				$address: String of address of the store
# 				$city : The city where the store is located in.
#				$state: The state where the store is located in.
#				$zipCode: The zipcode of the store.
#				$phone : The phone number of the store.
# Returns:
# 				The storeChainId and storeLocationId tuple as returned from the
#               database..
#
sub InsertStoreInformation {
    
    my $storeName = shift;
    my $storeWebsite = shift; 
    my $address = shift;
    my $city = shift;
    my $state = shift;
    my $zipCode = shift;
    my $phone   = shift;
	
	
    my $rows;
    my $storeChainId;
    my $storeLocationId;
		
    my $dbh;

    $dbh = ConnectToDatabase();

    # Find if the store already exists in the database, if it does not then insert it into the db	
	$storeChainId = GetScalarValue($dbh, "$STORE_CHAIN_TABLE_NAME", "id", "name = ?", $storeName);
    
    unless (defined($storeChainId)) {
		$rows = $dbh->do("INSERT INTO $STORE_CHAIN_TABLE_NAME (name, websiteUrl) VALUES (?,?)", undef, $storeName, $storeWebsite);		
		$storeChainId = GetLastInsertedId($dbh);
    }

    # Insert the store location information after making sure another store of the same zip code
    #does not exist
    eval { 
		$rows = $dbh->do(qq{INSERT INTO $STORE_LOCATION_TABLE_NAME 
								(storeChainId, address, city, state, zipCode, phone) 
								VALUES 
								($storeChainId, '$address', '$city', '$state', $zipCode, '$phone')});
    };

    if ($@) {

		$storeLocationId = GetScalarValue($dbh, "$STORE_LOCATION_TABLE_NAME", "id", 
						 "storeChainId = ? AND zipCode = ?", $storeChainId, $zipCode);
    }

    $dbh->disconnect();
    return ($storeChainId, $storeLocationId);
}


# A handy function to verify if the inserted data is in the database or not.
# Arguments:
#                  $storeName: Name of the store
#                  $zipCode: The zipcode of the store
#                  $specialsRef: The reference to a tuples (item, price) of specials.
# Returns a positive value if successful, otherwise negative value is returned.
#
sub VerifyInsertedSpecialsData {
	my $storeName = shift;
	my $zipCode = shift;
	my $specialsRef = shift;

	my $aRow;
	my $i;
	my $storeChainId;
	my $storeLocationId;
	
	my $dbh = ConnectToDatabase(); 

	$storeChainId = GetScalarValue($dbh, "$STORE_CHAIN_TABLE_NAME", "id", "name = ?", $storeName);
	
	unless ($storeChainId) {
		print STDERR "Cannot get the storeChainId\n";
		$dbh->disconnect();
		return -1;
	}

 	$storeLocationId = GetScalarValue($dbh, "$STORE_LOCATION_TABLE_NAME", "id", 
						 "storeChainId = ? AND zipCode = ?", $storeChainId, $zipCode);

	unless ($storeLocationId) {
		print STDERR "Cannot get the storeLocationId\n";
		$dbh->disconnect();
		return -2;
	}
						 
	for ($i = 0, $aRow = $specialsRef->[$i]; defined($aRow); $i++, $aRow = $specialsRef->[$i]) {
		my $result;
		$result = GetScalarValue($dbh, $SPEICAL_TABLE_NAME, "title", 
								"storeLocationId = ? AND title = ? AND price = ?" , 
								int($storeLocationId), $aRow->[0], $aRow->[1]);
		unless ($result) {
			print STDERR "storeLocationId = $storeLocationId, title = $aRow->[0], price = $aRow->[1] not found in the db";
			$dbh->disconnect();
			return -3;
		}				
	}
	return 0;	
}

# A handy function to test the GetScalarValue function
sub TestGetScalarValue() {

	my $failed = 0;
	print STDERR "Running test on GetScalarValue() function: ";
	
    my $dbh = ConnectToDatabase();
    
	# Check for a case when the value exists
    my $storeChainId = GetScalarValue($dbh, "$STORE_CHAIN_TABLE_NAME", "id", "name = ?", "QFC");
	
	TestCondition(defined($storeChainId), "\$storeChainId is undefined\n");
	
	print STDERR "Running [zipcode] test on GetScalarValue() function: ";
	
    my $storeLocationId =  GetScalarValue($dbh, "$STORE_LOCATION_TABLE_NAME", "id", 
																"storeChainId = ? AND zipCode = ?", $storeChainId, 98072);

	TestCondition(defined($storeLocationId),  "\$storeLocationId is undefined\n");
	
	$dbh->disconnect();
	    
}

# A handy function to test the GetStoreZipCodes function
sub TestZipCodeArray() {
	print STDERR  "Testing ZipCode array method: ";
	
    my @zipCodeArray = GetStoreZipCodes("QFC");
	
	TestCondition(scalar(@zipCodeArray) != 0, "");

}

# Handy function to test the connection to the database or
# the ConnectToDatabase() function
sub TestConnectToDb {
	print STDERR "Testing Connection to Db: ";
	
	my $dbh;
	eval {
		 $dbh = ConnectToDatabase();
	};
	
	TestCondition(! ($@),  $@);	
}
	
# A very handy function for outputting passed and failed based on a condition
# Arguments: 
#				$condition:	True or false
#				$errorMsg:	Error message to be printed if condition is false
#				$toDie:			Whether to call die() or not
# Return: 
#				None
# If condition is true, this method will print PASSED
# otherwise this method will print FAILED, appending the error message and
# if $toDie is true, the method will call die().
sub TestCondition {
	my $condition = shift;
	my $errorMsg = shift;
	my $toDie = shift;
	if ($condition) {
		print STDERR "PASSED!\n";
	}
	else {
		chomp($errorMsg) if ($errorMsg);		
		print STDERR "FAILED! ErrorMsg: \"$errorMsg\"\n";
		if ($toDie) { die(); }
	}
}

# A function to run all the test cases for this module.
sub RunDbSpecialTests() {
	
	print "\nRunning Tests for the db_specials\n";
	
	TestConnectToDb();
	TestGetScalarValue();
	TestZipCodeArray();
	
	
}

# A function that will check the passed in argument $error for common
# errors and provide user with better guidance.
# Normally you will pass in $@ as $error which is returned when die() is 
# called while running perl code in an eval block.
sub CheckEvalForError {
	my $error = shift;
	if ($error) {
		if ($error =~ m/DBI/) {
			print STDERR "********************************\n";
			print STDERR "Database returned an error.\n";
			print STDERR "* Make sure you have the right database credentials in file \"", GetOrSetDbConfigFile(),"\"\n";
			print STDERR "* Make sure the database is up and running.\n";
		}
		else {
			print STDERR "Seems like a store parser related error.\n";
			print STDERR "Maybe the store changed their interface.\n";
		}
	
		print STDERR "Please include all this message when filing a bug.\n";
		print STDERR "\n\n";
		print STDERR "The exact error is:\n\"$error\"\n";
		die("Exiting...");
	}
}

1;