#!/usr/local/bin/perl

use strict;
use warnings;
use AbstractParserFactory;
use db_specials;


my @TEST_CASES = ( "Automatically Get the zip codes: ",
									"Parser successfully parses the webpage: ",
									"Parser retrieves some specials: ",
									"Specials retrieved are valid: ",
									"Specials are correctly inserted into the database: ",
									"System test: GetZipCodeAndInsertSpecialsIntoDb(): "
								  );
								  
# If nothing or one thing except test is specified as an argument run theparser
# Otherwise run the tests							  
if ($#ARGV == 0 && $ARGV[0] =~ m/^test$/i) {
	RunTests();
}
elsif (scalar(@ARGV) == 0 || scalar(@ARGV) == 1) {
	eval {
		GetZipCodeAndInsertSpecialsIntoDb($ARGV[0]);
	};
	
	CheckEvalForError($@);

}
else {
	print STDERR "Usage\n";
	print STDERR "\tperl $0\n";
	print STDERR "\t\tTo fetch zipcodes from the db and insert the corresponding specials\n";
	print STDERR "\tperl $0 test\n";
	print STDERR "\t\tTo run unit and system testing on the store parser module\n";
	print STDERR "Also make sure you are running from a directory where all the dependent modules are present\n";
	print STDERR "Or the dependent modules are in the perl include path\n";
	print STDERR "See all the use/require statements to find the dependent modules\n";
}

# The main heart of the script.
# Gets the zip codes from the database for QFC and using each zip code
# inserts them into QFC.
# Arguments: $storeName (optional): The store name, defaults to "QFC" if undefined
#					 $verifyInsertedSpecials (optional): If this argument is passed in, then 
#                   this function will run in test mode and hence will not output anything
#					 This argument should always be second argument. 
#              
# Returns:
#                  Returns a positive number if the functions runs succesfully 
#                  and negative otherwise.
sub GetZipCodeAndInsertSpecialsIntoDb {

	
	my ($storeName, $verifyInsertedSpecials) = @_;
	
	$storeName = ($storeName) ? $storeName : "QFC";
	
    #my $storeName = "QFC";
	
    my @zipCodeArray = GetStoreZipCodes($storeName);
    
	if (scalar(@zipCodeArray) == 0) {
		print "No zip codes to test for $storeName.\n" unless (defined($verifyInsertedSpecials));
	}
	
    for my $zipCode (@zipCodeArray) {
	
		print "Getting specials for zipcode $zipCode\n" unless (defined($verifyInsertedSpecials));
		
		my $qfcParser = AbstractParserFactory->new($storeName);
		
		if (defined($verifyInsertedSpecials)) {
			$qfcParser->SetVerbosityLevel($AbstractParser::VERBOSITY_NONE);
		}
		else {
			$qfcParser->SetVerbosityLevel($AbstractParser::VERBOSITY_ALL);
		}
		
		my $allSpecialsRef = $qfcParser->GetSpecials($zipCode);
		
		InsertSpecialsIntoDb($storeName,
									  $zipCode,			 
									  $allSpecialsRef);
		
		if (defined($verifyInsertedSpecials)) {
			if (VerifyInsertedSpecialsData($storeName, $zipCode, $allSpecialsRef) < 0) {
				$verifyInsertedSpecials = -1;
			}
		}
    }
	
	return defined($verifyInsertedSpecials) ? $verifyInsertedSpecials : 0;
}


# A method to run all the tests
# Arguments: None
# Returns: None
# Relies on the @TEST_CASES global array					
sub RunTests {

	# Run the db_specials test cases
	RunDbSpecialTests();
	
	# Assuming QFC is in the database
	my $storeName = "QFC";	
	my @specialsReferences = ();
	my $currentTestId = 0;
	
	print STDERR "\nRunning test cases for the parser\n";
	print STDERR $TEST_CASES[$currentTestId++];
	
    my @zipCodeArray = GetStoreZipCodes($storeName);
    
	TestCondition(scalar(@zipCodeArray) != 0, "No zip code for QFC exiting...\n", 1);		
	
	print STDERR $TEST_CASES[$currentTestId++];
	
	eval {
		for my $zipCode (@zipCodeArray) {
			my $qfcParser = AbstractParserFactory->new("QFC");
			$qfcParser->SetVerbosityLevel($AbstractParser::VERBOSITY_NONE);
			my $allSpecialsRef = $qfcParser->GetSpecials($zipCode);
			push(@specialsReferences, $allSpecialsRef);
		}
	};
	
	TestCondition(! ($@), "QFC parser died with following error" . $@, 1);
	
	print STDERR $TEST_CASES[$currentTestId++];

	TestCondition(scalar(@specialsReferences) != 0, "No special references retrieved", 1);
		
	my $verifySpecialsFailed = 0;
	my $verifyInsertSpecialsFailed = 0;
	my $zipCodeIndex = 0;
	my $zipCode;
	for my $specialsRef (@specialsReferences) {
		my $i, my $aRow;
		for ($i = 0, $aRow = $specialsRef->[$i]; defined($aRow); $i++, $aRow = $specialsRef->[$i]) {
			if (not ($aRow->[0] && $aRow->[1])) {				
				$verifySpecialsFailed = "some of the specials are not valid";
			}
		}
		
		$zipCode = $zipCodeArray[$zipCodeIndex++];
		InsertSpecialsIntoDb($storeName, $zipCode, $specialsRef);
		
		if (VerifyInsertedSpecialsData($storeName, $zipCode, $specialsRef) < 0) {
			$verifyInsertSpecialsFailed = "the inserted specials do not match";
		}
	}
	
	print STDERR  $TEST_CASES[$currentTestId++];
	
	TestCondition(! ($verifySpecialsFailed), $verifySpecialsFailed);
	
	print STDERR  $TEST_CASES[$currentTestId++];
	
	TestCondition(! ($verifyInsertSpecialsFailed), $verifyInsertSpecialsFailed);
		
	print STDERR  $TEST_CASES[$currentTestId++];
	
	TestCondition(GetZipCodeAndInsertSpecialsIntoDb(undef, 1) >= 0, "");

}
 
