<?php

/**
 * Handles all site functionality except for login capabilities.
 *
 * @see index()
 * @package Controllers
 *
 */
class Search_Controller extends AbstractController
{
	/**
	 * The default search page. This page provides all site functionality except for login capabilities.
	 *
	 * In the following docs, a parameter with name $param refers to $_GET['param'] from the url parameters.
	 *
	 * If $type is 'special' or 'recipe':
	 * - If $type and $q are both set, this page performs a search for all items matching $q.
	 * - If $type and $id are both set, this page shows information on recipe or special matching the $id.
	 *
	 * If $type is 'favoritefood':
	 * - Searches for all specials matching the user's favorite foods.
	 * - No other parameters are required.
	 * - The user must be logged in.
	 *
	 * @param string $q GET variable. The query to search for, if $type is 'recipe' or 'special'.
	 * @param int $id GET variable. The recipe (if $type is 'recipe') or special (if $type is 'special') to view.
	 * @param string $type GET variable. Determines the page behavior. See above (method description) for details.
	 * @param string $zipCode GET variable. The zip code to use when viewing recipes, specials, etc. If not specified,
	 *                        defaults to the user's zipcode (stored from previous queries), or to a zipcode stored in
	 *                        session variables.
	 */
	function index()
	{
		$this->load->model('User_Model', 'user');

		$this->load->model('Recipe_Model', 'recipe');
		$this->load->model('Special_Model', 'special');
		$this->load->model('Favorites_Model', 'favorite');

		$this->view->assign('messages', Session_Handler::getMessages());
		$this->view->assign('errors', Session_Handler::getErrors());

		//
		// page logic
		$query = Url::get('q'); // search query
		$id = Url::get('id'); // if a specific recipe id or special id is given.
		$type = Url::get('type');
		$zipCode = Url::get('zipCode');

		$recipe_search = $type == 'recipe';
		$special_search = $type == 'special';
		$favorite_food_search = $type == 'favoritefood';

		$zipCode = (strncasecmp($zipCode, 'zipcode', 7) == 0) ? null : $zipCode;

		// preserve zipcode in session (might need it for subsequent recipe searches)
		if ($zipCode)
			$_SESSION['zipCode'] = $zipCode;

		// use the user's zip code if absolutely necessary.
		if (!$zipCode && $this->user->isLoggedIn())
			$zipCode = $this->user->zipCode();

		// use the session zip code as a last resort.
		if (!$zipCode && isset($_SESSION['zipCode']))
			$zipCode = $_SESSION['zipCode'];

		// set the user's zip code for future use.
		if ($this->user->isLoggedIn() && $zipCode)
			$this->user->zipCode($zipCode);

		$this->view->assign('user', $this->user->getArray());
		$this->view->assign('recipe_search', $recipe_search);
		$this->view->assign('special_search', $special_search);
		$this->view->assign('favorite_food_search', $favorite_food_search);
		$this->view->assign('view_recipe', $id && $recipe_search);
		$this->view->assign('view_special', $id && $special_search);


		// *************************************
		// Favorite Food Search (matching specials)
		if ($favorite_food_search) {

			$specials = array();
			$foods = $this->favorite->getSavedFoods($this->user->id());

			//
			// matching specials
			foreach ($foods as $food) {
				$matches = $this->special->getSpecialsByIngredient($food);
				$specials = $this->_mergeAssoc($specials, $matches, 'id');
			}

			// filter by zip code
			$specials = $this->special->filterByZipCode($specials, $zipCode);

			// get a list of associated recipes for specials.
			$recipes = array();
			foreach ($specials as $s) {
				$r = $this->recipe->getRecipesForSpecial($s['id']);
				$recipes[$s['id']] = $r;
			}

			$this->view->assign('specials', $specials);
			$this->view->assign('associated_recipes', $recipes);
		}

		// *************************************
		// Recipe Search
		if ($recipe_search) {
			$recipes = array();
			if ($id) {
				$recipe = $this->recipe->getRecipeById($id);
				$recipes[] = $recipe;
			} else {
				// recipe results.
				$recipes = $this->recipe->getRecipesByName($query);
				$recipes = $this->_mergeAssoc($recipes, $this->recipe->getRecipesByIngredient($query), 'id');
			}

			// get a list of associated specials for recipes.
			$specials = array();
			if ($zipCode) {
				foreach ($recipes as $r) {
					$s = $this->special->getSpecialsForRecipe($r['id'], $zipCode);
					$specials[$r['id']] = $s;
				}
			}

			$this->view->assign('recipes', $recipes);
			$this->view->assign('associated_specials', $specials);
		}

		// *************************************
		// Special Search
		if ($special_search) {
			$specials = array();
			if ($id) {
				$special = $this->special->getSpecialById($id);
				$specials[] = $special;
			} else {
				// special results. (search both by ingredient and special title and special description)
				$specials = $this->special->getSpecialsByName($query);
				$specials = $this->_mergeAssoc($specials, $this->special->getSpecialsByIngredient($query), 'id');
				$specials = $this->special->filterByZipCode($specials, $zipCode);
			}

			// get a list of associated recipes for specials.
			$recipes = array();
			foreach ($specials as $s) {
				$r = $this->recipe->getRecipesForSpecial($s['id']);
				$recipes[$s['id']] = $r;
			}

			$this->view->assign('specials', $specials);
			$this->view->assign('associated_recipes', $recipes);
		}

		// *************************************
		// User Favorite Foods
		if ($this->user->isLoggedIn()) {
			$special_ids = $this->favorite->getSavedSpecialIds($this->user->id());
			$favorite_specials = $this->favorite->getSavedSpecials($this->user->id(), $this->special);
			$favorite_specials = !empty($special_ids)
				? array_combine($special_ids, $favorite_specials)
				: array();

			$recipe_ids = $this->favorite->getSavedRecipeIds($this->user->id());
			$favorite_recipes = $this->favorite->getSavedRecipes($this->user->id(), $this->recipe);
			$favorite_recipes = !empty($recipe_ids)
				? array_combine($recipe_ids, $favorite_recipes)
				: array();

			$this->view->assign('favorite', array(
				'specials' => $favorite_specials,
				'recipes' => $favorite_recipes,
				'foods' => $this->favorite->getSavedFoods($this->user->id()),
			));
		}

		// Available zip codes
		$this->view->assign('availableZipCodes', $this->special->getAllZipCodes());
		
		// *************************************
		// Display the results (regardless of type).
		$this->view->display('search.html');
	}

	/**
	 * Merges array1 and array2 using the associative key 'key'.
	 * Removes duplicate values.
	 *
	 * @param array $array1 The first array.
	 * @param array $array2 The second array.
	 * @param mixed $key The key to use when merging $array1 and $array2.
	 *
	 * @private
	 */
	function _mergeAssoc($array1, $array2, $key)
	{
		return $this->_distinct(array_merge($array1, $array2), $key);
	}

	/**
	 * Returns a list of the distinct values of $array based on $key.
	 * I.e., guarantees no two $array[$key] have the same value.
	 *
	 * @param array $array The array from which to get distinct values
	 * @param mixed $key The key to use to get distinct values.
	 *
	 * @private
	 */
	function _distinct($array, $key)
	{
		$result = array();
		foreach ($array as $a) {
			if (!$this->_hasKeyVal($result, $key, $a[$key])) {
				$result[] = $a;
			}
		}

		return $result;
	}

	/**
	 * Returns whether array has a key with the given value
	 *
	 * @param array $array
	 * @param mixed $key The key in $array to use when searching.
	 * @param mixed $value The value to search for using condition: ($array[$key] == $value).
	 *
	 * @private
	 */
	function _hasKeyVal($array, $key, $value)
	{
		foreach ($array as $a) {
			if ($a[$key] == $value) {
				return true;
			}
		}

		return false;
	}
}

?>